% Multiplies two matrices with three different methods and compares
% performances

% Defining 2 random matrices of random (compatible) sizes
n1=400;n2=500;n3=800;
% We use randi() to get randoms integers so that numerical precision is not
% an issue when comparing results from different methods
% Big sizes are better to compare performances
A=randi([-100 100],[n1 n2]);
B=randi([-100 100],[n2 n3]);
% Pre-allocating C saves time and memory
C=zeros(n1,n3);

% tic ... toc measures time spent between the two lines
tic
% Built-in matrix multiplication method
C0=A*B;
toc

tic
% Naive element-by-element method straight from the definition
for ii=1:size(A,1)
   for jj=1:size(B,2)
      s=0;
      for kk=1:size(A,2)
         s=s+A(ii,kk)*B(kk,jj);
      end
      C(ii,jj)=s;
   end
end
% Checking if the result is correct
isequal(C,C0)
toc


tic
% Direct scalar products of rows and columns
for ii=1:size(A,1)
   for jj=1:size(B,2)
      C(ii,jj)=A(ii,:)*B(:,jj);
   end
end
% Checking if the result is correct
isequal(C,C0)
toc