
clear

%% COORDINATE VECTORS AND MATRICES
x_line = linspace(-2,2,25);
y_line = linspace(-2,2,25);

[x, y] = meshgrid(x_line, y_line);


%% CALCULATE FIELD

q1 = +1; % charge of particle 1
q2 = +1; % charge of particle 2

eps = 0.1; % trick to avoid the divergence at r = 0

% Potential = 1/r
Potential = q1 ./ sqrt(eps + (x+1).^2 + y.^2) + q2 ./ sqrt(eps + (x-1).^2 + y.^2);

% Electric Field = - gradient(Potential)
[EField_x,EField_y] = gradient(-Potential);


%% PLOTS
figure(1)
clf

subplot(2,2,1)
quiver(x, y, EField_x, EField_y)
axis([x_line(1) x_line(end) y_line(1) y_line(end)])
axis square
title('Electric field due to two charges')
xlabel('x')
ylabel('y')

subplot(2,2,2)
streamslice(x, y, EField_x, EField_y)
axis([x_line(1) x_line(end) y_line(1) y_line(end)])
axis square
title('Electric field due to two charges')
xlabel('x')
ylabel('y')

subplot(2,2,3)
pcolor(x,y,Potential)
shading interp
colormap hot
axis([x_line(1) x_line(end) y_line(1) y_line(end)])
axis square
title('Electric potential due to two charges')
xlabel('x')
ylabel('y')

subplot(2,2,4)
hold on
pcolor(x,y,Potential)
shading interp
colormap hot
streamslice(x, y, EField_x, EField_y)
axis([x_line(1) x_line(end) y_line(1) y_line(end)])
axis square
title('Electric field and potential due to two charges')
xlabel('x')
ylabel('y')
hold off