%% -- Segmenting multiple images ------------------------------------------
% RMT November 2015 
%
% This script is for importing multiple images from the current path. 
% We'll also use some graphical inputs to remove the background. 
% There will be extra stuff in this script for reference after the class 
% (pop up windows etc.)
%
%%-------------------------------------------------------------------------

clear
close all
iptsetpref('ImshowBorder', 'tight')

%% Specify paths and import files
% This is mainly for reference; you can copy & paste these lines into other
% scripts as necessary.

% Note that we're assuming the files are names 'C1.jpg', 'C2.jpg', etc.
chpar_ans = input('Change import parameters? y/n [n] ', 's');
if chpar_ans == 'y'
    path_prompt={'Import path:','Export path:'};
    path_dlg_title='Select import and export paths (Can include beginning of filenames, assuming numerical ending)';
    path_num_lines=4;
    path_default={'Year 2/SkillPill/Cells/C','Year 2/SkillPill/Cells/Results/'};
    options.Resize='on';
    path=inputdlg(path_prompt,path_dlg_title,path_num_lines,path_default,options);
    import_path=char(path(1));
    export_path=char(path(2));
    clear path path_prompt path_dlg_title path_num_lines path_default
    % Because sometimes making your own pop-up is quite satisfying.
else    
    import_path = 'Year 2/SkillPill/Cells/C';
    % If the files are in your current path you can just use the filename
    % and extension, or the folder path from your current path.
    export_path = 'Year 2/SkillPill/Cells/Results/';
    % Matlab will create a folder for you if it doesn't exist.
end
clear chpar_ans

% Import images into one array

image_array = rgb2gray(imread(strcat(import_path, '1.jpg')));

for n = 2:length(dir(strcat(import_path,'*.jpg')));
    
    image_array(:,:,n) = rgb2gray(imread(strcat(import_path, num2str(n), '.jpg')));
    
end    

image_array = im2double(image_array);
% Using images of type uint8 will cause problems later on - combining
% large(ish) integer values (gradient intercept) with decimal values
% (gradient) results in a blank image.

%% Select areas to remove background gradient
% For this image type, there are numerous automatic ways in which we could
% do this (see documentation for examples), but here we will use another
% example of graphical input, imcrop(). Under what circumstances might this
% be useful?

% Also note that the disks are deliberately different shades which might
% trip up some programs without proper thresholding (if using automatic
% methods).

% We'll make a plane fit for the first image and then apply it to the each
% image contained in image_array.

figure, imshow(image_array(:,:,1))
display('Select background area near top of image to be fitted. Double click to confirm.')
[croptop, rec] = imcrop(gcf); 
% rectangle contains xmin, ymin, width, height values for position of 
% cropped image in original - note that xmin and ymin are at the top left
% of the area selected.

itop=[rec(1),rec(1)+rec(3),rec(2),rec(2)+rec(4)];
% these elements are then xmin, xmax, ymin, ymax in image coordinates.
itop=[ceil(itop(1)) floor(itop(2)) ceil(itop(3)) floor(itop(4))]; 
% this rounds the elements such that we select pixels contained in the
% cropped area.

%figure, imshow(croptop)

% To illustrate again why we have to be careful with graphical input
% coordinates and image coordinates, let's indicate the location of each
% corner of the cropped image onto the original.
figure(1)
hold on
%plot(itop(1),itop(3),'+')
%plot(itop(1),itop(4),'+')
%plot(itop(2),itop(3),'+')
%plot(itop(2),itop(4),'+')
text(itop(1),itop(3),'1')
text(itop(2),itop(3),'2')
text(itop(1),itop(4),'3')
text(itop(2),itop(4),'4')
hold off
% Unlike with the ginput() command, here we are using a function from the
% image processing toolbox, which means that the rectangle coordinates
% xmin, ymin given are in spatial image coordinates (i.e. from the top left
% corner). The width and height are as you'd expect.

display('Select background area near base of image to be fitted. Double click to confirm.')
[cropbot, rec] = imcrop(gcf); 

ibot=[rec(1),rec(1)+rec(3),rec(2),rec(2)+rec(4)];
ibot=[ceil(ibot(1)) floor(ibot(2)) ceil(ibot(3)) floor(ibot(4))];

figure(1)
hold on
text(ibot(1),ibot(3),'1')
text(ibot(2),ibot(3),'2')
text(ibot(1),ibot(4),'3')
text(ibot(2),ibot(4),'4')
hold off

%% Plane fit data

[plane_gradient, plane_intercept] = linearplanefit(croptop, cropbot, itop, ibot);
% See the function linearplanefit.m

planefit_matrix = zeros(size(image_array(:,:,1)));
for n = 1:size(image_array(:,:,1),1)
    planefit_matrix(n,:) = (plane_gradient * n) + plane_intercept;
end
%figure, imshow(planefit_matrix)
planefit_matrix = planefit_matrix - min(min(planefit_matrix));
% Offset for display purposes.
%figure, imshow(planefit_matrix)

for m = 1:size(image_array,3)
    I_planed = image_array(:,:,m) - planefit_matrix;
    I_planed = I_planed - min(min(I_planed));
    image_array(:,:,m) = I_planed;
    %figure, imshow(I_planed)
end

clear m n croptop cropbot rec plane_intercept plane_gradient planefit_matrix itop ibot I_planed
close all
